package main

import (
	"sort"
	"strconv"
	"strings"
)

// StringSlice ...
type StringSlice []string

func (strs StringSlice) Len() int {
	return len(strs)
}

func (strs StringSlice) Swap(i, j int) {
	strs[i], strs[j] = strs[j], strs[i]
}

func (strs StringSlice) Less(i, j int) bool { // Less is "More"
	len1, len2 := len(strs[i]), len(strs[j])
	for k := 0; k < len1 && k < len2; k++ {
		if strs[i][k] != strs[j][k] {
			return !(strs[i][k] < strs[j][k])
		}
	}
	if len1 < len2 {
		for k := len1; k < len2; k++ {
			if strs[i][len1-1] != strs[j][k] {
				return !(strs[i][len1-1] < strs[j][k])
			}
		}
	} else if len1 > len2 {
		for k := len2; k < len1; k++ {
			if strs[i][k] != strs[j][len2-1] {
				return !(strs[i][k] < strs[j][len2-1])
			}
		}
	}
	return !false
}

func largestNumber(nums []int) string {
	strs := make([]string, 0)
	for _, n := range nums {
		strs = append(strs, strconv.Itoa(n))
	}
	sort.Sort(StringSlice(strs))
	var sb strings.Builder
	for _, s := range strs {
		if sb.Len() == 0 && s == "0" {
			continue // No leading zero
		}
		sb.WriteString(s)
	}
	if sb.Len() == 0 {
		return "0"
	}
	return sb.String()
}

// func main() {
// 	println(largestNumber([]int{
// 		3, 30, 34, 5, 9}))
// 	println(largestNumber([]int{
// 		824, 938, 1399, 5607, 6973, 5703, 9609, 4398, 8247}))
// }
